; Original file Copyright (c) 2001, David H. Hovemeyer <daveho@cs.umd.edu>
; $Revision: 1.16 $

; Copyright (C) 1991, 1992 Linus Torvalds
; adapted for Kernel Toolkit by Luigi Sgro

%include "../include/defs.s"

[BITS 16]
[ORG 0x0]

start_setup:

	; Redefine the data segment so we can access variables
	; declared in this file.
	mov	ax, SETUPSEG
	mov	ds, ax

.setup:
	mov si, strSetup
	call DisplayString

	; Use int 15h to find out size of extended memory in KB.
	; Extended memory is the memory above 1MB.  So by
	; adding 1MB to this amount, we get the total amount
	; of system memory.  We can only detect 64MB this way,
	; but that's OK for now.
	xor	eax, eax
	mov	ah, 0x88
	int	0x15
	add	eax, 1024	; 1024 KB == 1 MB
	mov	[mem_size_kbytes], eax

	mov si, strMemory
	call DisplayString
	mov edx, eax
	call PrintHex8
	call PrintNL

	mov si, strFloppy
	call DisplayString
	; Kill the floppy motor.
	call	Kill_Motor

	; wait code removed, shouldnt be needed anymore, no errors
	;    have been found in the boot code so we can assume all
	;    errors are in the kernel itself.
;	mov si, strWait
;	call DisplayString
;	; wait for keypress
;	mov ah, 0x10        ;get character
;    int 0x16

	; Block interrupts, since we can't meaningfully handle them yet
	; and we no longer need BIOS services.
	cli

	; Set up IDT and GDT registers
	lidt	[IDT_Pointer]
	lgdt	[GDT_Pointer]

	; Initialize the interrupt controllers, and enable the
	; A20 address line
	call	Init_PIC
	call	Enable_A20

	; Switch to protected mode!
	mov	ax, 0x01
	lmsw	ax

	; Jump to 32 bit code.
	; NASM probably has a way to express this using
	; instruction mnemonics, but I don't know how, so I
	; just copied this stuff from Linux/KTK :-)
	db 0x66			; operand size override prefix
	db 0xea			; jmp opcode
	dd (SETUPSEG<<4) + setup_32 ; want to jump here
	dw 1<<3 ; ...in the kernel code segment

[BITS 32]
setup_32:

	; set up data segment registers
	mov	ax, 2<<3
	mov	ds, ax
	mov	es, ax
	mov	fs, ax
	mov	gs, ax
	mov	ss, ax

	; Create the stack for the initial kernel thread, just below the kernel
	mov	esp, (0xEFFF)

	; Build BootInfo struct on stack.
	; Note that we push the fields on in reverse order,
	; since the stack grows downwards.
	mov	eax, [(SETUPSEG<<4)+mem_size_kbytes]
	push	eax		; memSizeKB
	push	dword 8		; bootInfoSize

	; Pass pointer to Boot_Info struct as argument to kernel
	; entry point.
	push	esp

	; Push return address to make this look like a call
	; This DOES work
	push	dword (SETUPSEG<<4)+.returnAddr

	; Far jump into kernel into Funx Main in main.cxx
	jmp	1<<3:0x10000 

.returnAddr:
	; We shouldn't return here.
.here:	jmp .here

[BITS 16]

; Kill the floppy motor.
; This code was shamelessly stolen from Linux.
Kill_Motor:
	mov	dx, 0x3f2
	xor	al, al
	out	dx, al
	ret

Init_PIC:
	; Initialize master and slave PIC!
	mov	al, ICW1
	out	0x20, al		; ICW1 to master
	call	Delay
	out	0xA0, al		; ICW1 to slave
	call	Delay
	mov	al, ICW2_MASTER
	out	0x21, al		; ICW2 to master
	call	Delay
	mov	al, ICW2_SLAVE
	out	0xA1, al		; ICW2 to slave
	call	Delay
	mov	al, ICW3_MASTER
	out	0x21, al		; ICW3 to master
	call	Delay
	mov	al, ICW3_SLAVE
	out	0xA1, al		; ICW3 to slave
	call	Delay
	mov	al, ICW4
	out	0x21, al		; ICW4 to master
	call	Delay
	out	0xA1, al		; ICW4 to slave
	call	Delay
	mov	al, 0xff		; mask all ints in slave
	out	0xA1, al		; OCW1 to slave
	call	Delay
	mov	al, 0xfb		; mask all ints but 2 in master
	out	0x21, al		; OCW1 to master
	call	Delay
	ret

; Linux uses this code.
; The idea is that some systems issue port I/O instructions
; faster than the device hardware can deal with them.
Delay:
	jmp	.done
.done:	ret

; Enable the A20 address line, so we can correctly address
; memory above 1MB.
Enable_A20:
	mov	al, 0xD1
	out	0x64, al
	call	Delay
	mov	al, 0xDF
	out	0x60, al
	call	Delay
	ret


%include "../include/util.s"

; ----------------------------------------------------------------------
; Setup data
; ----------------------------------------------------------------------

mem_size_kbytes: dd 0


; ----------------------------------------------------------------------
; The GDT.  Creates flat 32-bit address space for the kernel
; code, data, and stack.  Note that this GDT is just used
; to create an environment where we can start running 32 bit
; code.  The kernel will create and manage its own GDT.
; ----------------------------------------------------------------------
GDT:
	; Descriptor 0 is not used
	dw 0
	dw 0
	dw 0
	dw 0

	; Descriptor 1: kernel code segment
	dw 0xFFFF	; bytes 0 and 1 of segment size
	dw 0x0000	; bytes 0 and 1 of segment base address
	db 0x00		; byte 2 of segment base address
	db 0x9A		; present, DPL=0, non-system, code, non-conforming,
			;   readable, not accessed
	db 0xCF		; granularity=page, 32 bit code, upper nibble of size
	db 0x00		; byte 3 of segment base address

	; Descriptor 2: kernel data and stack segment
	; NOTE: what Intel calls an "expand-up" segment
	; actually means that the stack will grow DOWN,
	; towards lower memory.  So, we can use this descriptor
	; for both data and stack references.
	dw 0xFFFF	; bytes 0 and 1 of segment size
	dw 0x0000	; bytes 0 and 1 of segment base address
	db 0x00		; byte 2 of segment base address
	db 0x92		; present, DPL=0, non-system, data, expand-up,
			;   writable, not accessed
	db 0xCF		; granularity=page, big, upper nibble of size
	db 0x00		; byte 3 of segment base address

; GDT initialization stuff
NUM_GDT_ENTRIES equ 3		; number of entries in GDT
GDT_ENTRY_SZ equ 8		; size of a single GDT entry

GDT_Pointer:
	dw 512	; limit
	dd (SETUPSEG<<4) + GDT		; base address

IDT_Pointer:
	dw 0
	dd 00

; strings
	strSetup db "In setup code:", 0x0D, 0x0A, 0x00
	strMemory db "BIOS reports memory size in KB is: 0x", 0x00
	strFloppy db "Killing floppy motor", 0x0D, 0x0A, 0x00
;	strWait db "Ready to move into kernel, press any key to continue", 0x0D, 0x0A, 0x00
EndSetupText:
FillSector  times (512 - (EndSetupText - start_setup)) db 0
