
#ifndef _DEVICE_H_
#define _DEVICE_H_

/* device structure */
struct __k_dev;
typedef struct __k_dev k_dev;

/* function pointer types */
typedef int_t (*fnOpen_t)(fileh f);
typedef int_t (*fnClose_t)(fileh f);
typedef int_t (*fnRead_t)(fileh f, char_t* ch);
typedef int_t (*fnWrite_t)(fileh f, char_t ch);
typedef int_t (*fnRead_R_t)(fileh f, void* pBuf, ulong_t rec);
typedef int_t (*fnWrite_R_t)(fileh f, void* pBuf, ulong_t rec);
typedef int_t (*fnIoctl_t)(fileh f, ulong_t request, void* arg);

/* imp of device structure */
struct __k_dev
{
	ulong_t			flags;
	ulong_t			ulRecordSize;	/* 0 for char devices */
	
	fnOpen_t		fnOpen;
	fnClose_t		fnClose;
	
	fnRead_t		fnRead;
	fnRead_R_t		fnRead_R;
	
	fnWrite_t		fnWrite;
	fnWrite_R_t		fnWrite_R;
	
	fnIoctl_t		fnIoctl;
};


/* errors */
#define DEV_SUCCESS 				(0)

#define DEV_ERR_INVALID				(-10)
#define DEV_ERR_ALREADY_CLOSED		(-20)
#define DEV_ERR_TABLE_FULL			(-100)
#define DEV_ERR_NO_ENTRY			(-110)
#define DEV_ERR_INVALID_REQUEST		(-200)
#define DEV_ERR_NO_RECORD			(-250)
#define DEV_ERR_NO_DATA				(-260)

/* introduced in demo6 */
#define DEV_ABORT					(-1)
#define DEV_ERR_BAD_PARAM			(-270)

/* device.c functions */
fileh OpenDevice(ulong_t dev);
k_dev* GetDevice(ulong_t dev);
int_t RegisterDevice(ulong_t dev, k_dev* pDev);
int_t UnregisterDevice(ulong_t dev);

/* dev_tbl.c functions */
void InitDeviceTable();
k_dev* device_tbl_lookup(ulong_t dev);
int_t device_tbl_add(ulong_t dev, k_dev* pDev);
int_t device_tbl_remove(ulong_t dev);


#endif /* !_DEVICE_H_ */
