
#ifndef _GDT_H_
#define _GDT_H_
#include <defs.h>
#include <stdtypes.h>
#include <memory.h>
/* The general format of a segment descriptor, taken from GeekOS */
typedef struct __gdtEntry {
	word_t sizeLow				 __attribute__((packed)) ;
	dword_t baseLow			: 24 __attribute__((packed)) ;
	dword_t type			: 4  __attribute__((packed)) ;
	dword_t system			: 1  __attribute__((packed)) ;
	dword_t dpl				: 2  __attribute__((packed)) ;
	dword_t present			: 1  __attribute__((packed)) ;
	dword_t sizeHigh		: 4  __attribute__((packed)) ;
	dword_t avail			: 1  __attribute__((packed)) ;
	dword_t reserved		: 1  __attribute__((packed)) ; /* set to zero */
	dword_t dbBit			: 1  __attribute__((packed)) ;
	dword_t granularity		: 1  __attribute__((packed)) ;
	half_t baseHigh				 __attribute__((packed)) ;
} gdtEntry;
#define gdtSetSize(g, sz) (g)->sizeLow = (sz & 0xFFFF); (g)->sizeHigh = ((sz >> 16) & 0x0F)
#define gdtSetBase(g, b) (g)->baseLow = (b & 0xFFFFFF); (g)->baseHigh = (b >> 24)
/* This is defined in int.s, and needs moving somewhere else */
/* The parameter should consist of 16 bit base, */
/* followed by 32 bit base address, describing the IDT. */
extern  void gdtLoad(word_t* limitAndBase);

/* public functions from gdt.c */
void InitGDT();	
void InitLDT();
gdtEntry* gdtGet(byte_t entry);
void gdtPut(byte_t entry, gdtEntry* desc);
void gdtRemove(byte_t entry);
gdtEntry* gdtGetFree(byte_t* entry);
extern  byte_t gdtAdd(byte_t type,
			ulong_t base ,
			ulong_t size ,
			byte_t system ,
			byte_t granularity ,
			byte_t dbBit ,
			byte_t dpl );

void gdtSet(byte_t entry,
			byte_t type,
			ulong_t base ,
			ulong_t size ,
			byte_t system ,
			byte_t granularity ,
			byte_t dbBit ,
			byte_t dpl );
#define GDTSET(x,y) gdtSet(x,y,0,0xfffff,1,1,1,0)
#define GDTADD(x) gdtAdd(x,0,0xfffff,1,1,1,0)
#define GDTBASE(x) (((x)->baseLow)|((x)->baseHigh)<<24)
/* start at 5k, span 1k, finish at 6k (below IDT)  */
#define GDT_LIMIT (0x400)

#define GDT_NUM_ENTRIES (GDT_LIMIT >> 3)

extern  dword_t getLinear (dword_t selector, dword_t offset);
#endif /* !_GDT_H_ */

