
[BITS 32]

%include "../include/symbol.s"

; include this for KERNEL_CS
%include "../include/defs.s"

EXPORT idtLoad
EXPORT gdtLoad

EXPORT DefInterrupt0
EXPORT DefInterrupt1
EXPORT DefInterrupt2
EXPORT DefInterrupt3
EXPORT DefInterrupt4
EXPORT DefInterrupt5
EXPORT DefInterrupt6
EXPORT DefInterrupt7
EXPORT DefInterrupt8
EXPORT DefInterrupt9
EXPORT DefInterrupt10
EXPORT DefInterrupt11
EXPORT DefInterrupt12
EXPORT DefInterrupt13
EXPORT DefInterrupt14
EXPORT DefInterrupt15
EXPORT DefInterrupt16
EXPORT DefInterrupt17
EXPORT DefInterrupt18
EXPORT DefInterrupt19

EXPORT KeyboardInterrupt
EXPORT FloppyInterrupt
EXPORT DebuggerInterrupt


IMPORT ProcessKeycode
IMPORT doDebuggerInt
IMPORT progData
IMPORT DefInt
IMPORT getLinear
IMPORT DoFloppyInterrupt
IMPORT displayTss

;note that ss , cs, eflags,eip are not saved , if need to be saved ,that is done by the 386 itself.
%macro save_state 0
	nop;
	nop;
	pushad ; ax,cx,dx,bx,sp old,bp,si,di  in that order
		;pushing segments in this order ds,es,fs,gs
	xor eax,eax
	mov ax,ds
	push eax
	
	xor eax,eax
	mov ax,es
	push eax
	
	xor eax,eax
	mov ax,fs
	push eax
	
	xor eax,eax
	mov ax,gs
	push eax	
	
%endmacro
%macro  move_to_kernel_ds 0
	mov ax,KERNEL_DS
	mov ds,ax
	mov es,ax
	mov fs,ax
	mov gs,ax
%endmacro
%macro restore_state 0
	pop eax
	mov gs,ax
	
	pop eax
	mov fs,ax
	
	pop eax
	mov es,ax
	
	pop eax
	mov ds,ax
	
	popad
%endmacro
%macro _DefInterrupt 1
EXPORT DefInterrupt%1
align 8
DefInterrupt%1:
	save_state
	move_to_kernel_ds
	push %1
	call DefInt
	add esp,4
	restore_state
	iret
%endmacro
; ----------------------------------------------------------------------
; Code
; ----------------------------------------------------------------------

[SECTION .text]


; Load IDTR with 6-byte pointer whose address is passed as
; the parameter. 
align 8
idtLoad:
	mov	eax, [esp+4]
	lidt [eax]
	ret

; Load GDTR with 6-byte pointer whose address is passed as
; the parameter. This really belongs somewhere else, but for
; now it is here.
align 8
gdtLoad:
	mov eax, [esp+4]
	lgdt [eax]

	; we know that all the segment registers are
	; correct because they were set during setup

	; now do a far jump to reload everything
	jmp KERNEL_CS:.here

.here:
	ret
align 8
DefInterrupt0:
	push 0
	call DefInt
	add esp,4
	iret
align 8
DefInterrupt1:
	push 1
	call DefInt
	add esp,4
	iret
align 8
DefInterrupt2:
	push 2
	call DefInt
	add esp,4
	iret
align 8
DefInterrupt3:
	push 3
	call DefInt
	add esp,4
	iret
align 8
DefInterrupt4:
	push 4
	call DefInt
	add esp,4
	iret
align 8
DefInterrupt5:
	push 5
	call DefInt
	add esp,4
	iret
align 8
DefInterrupt6:
	push 6
	call DefInt
	add esp,4
	iret
align 8
DefInterrupt7:
	push 7
	call DefInt
	add esp,4
	iret
align 8
DefInterrupt8:
	push 8
	call DefInt
	add esp,4
	iret
align 8
DefInterrupt9:
	push 9
	call DefInt
	add esp,4
	iret
align 8
DefInterrupt10:
	push 10
	call DefInt
	add esp,4
	iret
align 8
DefInterrupt11:
	push 11
	call DefInt
	add esp,4
	iret
align 8
DefInterrupt12:
	push 12
	call DefInt
	add esp,4
	iret
align 8
DefInterrupt13:
	push 13
	call DefInt
	add esp,4
	iret
align 8
DefInterrupt14:
	push 14
	call DefInt
	add esp,4
	iret
align 8
DefInterrupt15:
	push 15
	call DefInt
	add esp,4
	iret
align 8
DefInterrupt16:
	push 16
	call DefInt
	add esp,4
	iret
align 8
DefInterrupt17:
	push 17
	call DefInt
	add esp,4
	iret
align 8
DefInterrupt18:
	push 18
	call DefInt
	add esp,4
	iret
align 8
DefInterrupt19:
	push 19
	call DefInt
	add esp,4
	iret

align 8
KeyboardInterrupt:
	push ax
	push dx
	mov dx, 0x64
	xor ax, ax
	in al, dx		; get keyboard status
	and al, 1		; check if there is some data ready
	jz .done

	mov dx, 0x60
	xor ax, ax
	in al, dx		; get data
	push ax			; push onto stack as an argument
	call ProcessKeycode
	pop ax			; put things back how they were

.done
	mov dx, 0x20
	mov al, 0x61
	out dx, al		; EOI to master PIC

	pop dx
	pop ax
	iret

align 8
FloppyInterrupt:
	save_state
	move_to_kernel_ds
	call DoFloppyInterrupt
	mov dx, 0x20
	mov al, 0x66	; IRQ 6
	out dx, al		; EOI to master PIC
	restore_state
	iret
align 8
DebuggerInterrupt:
		;stack before has ss,esp,eflags,cs,eip in this order 
		; the subsequent sp value pushed hence points to the eip
	
	save_state
	move_to_kernel_ds
	push esp	;points to the just pushed in gs above

	xor eax,eax
	mov ax,ss
	push eax
	
	call getLinear
	
	mov [progData],eax
	
	add esp , 8
	
	mov eax,dr6 ; save debug reg
	push ebx
	xor ebx,ebx
	mov dr6,ebx	;clear dr6
	pop ebx
	push eax; this will be some garbage right now
	call doDebuggerInt ; this does some printing and switches to the kernel task .
	add esp,4
	restore_state	
	iret ; this shll switch back to the original stack.

