#include <stdtypes.h>

#include <kalloc.h>
/*#include <bget.h> */
#include <sys.h>
#include <gdt.h>
#include <stdtypes.h>
#include <asm.h>
#include <video.h>

/*/////////////////////////////////////////////////////////////////// */
/* pgAlloc() and pgRelease(). */

/* This makes our default bitmap 96 bytes in size. */

/* The maximum amount of space that is guarenteed for the bitmap is 8k. */
/*   See the memory map file, "Memory" */

/* Initialise the allocator */
void InitPG()
{
	memset((ptr_t)BITMAP_START, (byte_t)0, BITMAP_SIZE);
}
void dummy(){}
int _ctr_alloc=0;
int _ctr_release=0;
ptr_t pgAlloc()
{
	_ctr_alloc++;
	/*printk("\nCalling pgAlloc "); */
	count_t n = 0;
	count_t x;
	byte_t* p = (byte_t*)BITMAP_START;

	while (n < BITMAP_SIZE) /* loop through each byte of the page table */
	{
		x = 0;
		while (x < 8)
		{
			if (!(*p & (1 << x)))	/* check allocated flag */
			{
				*p |= (1 << x);
				return (ptr_t)(POOL_START + PAGE_SIZE * (x + n * 8));
			}
			++x;
		}
		++n;
		++p;
	}
	panic("Out of memory");
	return 0; /* never actually happens, just gets rid of warnings */
}
byte_t installSegmentAt(int idx,ptr_t base,size_t size,enum purpose dc,enum privilege ring){
	/*install this in the gdt. */
	byte_t gra;
	dword_t limit;
	gdtEntry* entry=gdtGet(idx);
	if(!entry){
		VidWrite("Cannot find a free gdt entry to install the segment");
		do_nothing();/*never returns */
	}
	memset((ptr_t)entry,0,sizeof(gdtEntry));
	if (size > 0x100000) { limit = size/0x1000; gra = 1; }
	else { limit = size-1; gra = 0; }
	entry->sizeLow= limit&0xFFFF;
	entry->baseLow = ((dword_t)base)&0xFFFFFF;
	entry->present = 1;
	entry->granularity=gra;
	entry->reserved=0;
	entry->system=(dc==TSS || ring==SYSTEM?0:1);
	entry->dpl=(ring==USER?3:0);
	entry->type=(dc==CODE?0xb:(dc==TSS?0x9:0x3));
	entry->dbBit=1;
	entry->avail=0;
	entry->baseHigh = ((dword_t)base)>>24&0xFF;
}
byte_t installSegment(ptr_t base,size_t size,enum purpose dc,enum privilege ring){
	/*install this in the gdt. */
	byte_t freeSlot;
	gdtEntry* entry=gdtGetFree(&freeSlot);
	byte_t gra;
	dword_t limit;
	if(!entry){
		VidWrite("Cannot find a free gdt entry to install the segment");
		do_nothing();/*never returns */
	}
	memset((ptr_t)entry,0,sizeof(gdtEntry));
	if (size > 0x100000) { limit = size/0x1000; gra = 1; }
	else { limit = size-1; gra = 0; }
	entry->sizeLow= limit&0xFFFF;
	entry->baseLow = ((dword_t)base)&0xFFFFFF;
	entry->present = 1;
	entry->granularity=gra;
	entry->reserved=0;
	entry->system=(dc==TSS || ring==SYSTEM?0:1);
	entry->dpl=(ring==USER?3:0);
	entry->type=(dc==CODE?0xb:(dc==TSS?0x9:0x3));
	entry->dbBit=1;
	entry->avail=0;
	entry->baseHigh = ((dword_t)base)>>24&0xFF;
	return freeSlot;	
}
void releaseSegment(byte_t sel){
	gdtRemove(sel);	
}
void pgRelease(void* pg) /*if continuos cleear each seperately . */
{
	iptr_t page = (iptr_t)pg;	/* use a simple type */
	count_t nbyte;
	count_t nbit;
	byte_t* p;

	/* convert the pointer to a page number */
		page -= POOL_START;
		page /= PAGE_SIZE;
	/* now split */
		nbit = (count_t)(page & 0x07);
		nbyte = (page >> 3);
		p = (byte_t*)(BITMAP_START + nbyte);
	/*printk("\nCalled pgRelease to free page no %d",nbit+(nbyte<<3)); */
	_ctr_release++;
	if (*p & (1 << nbit))
		*p &= ~(1 << nbit); /* clear the allocated flag */
	else
		panic("Trying to free an unallocated page");
}
void pgRelease_c(ptr_t start,int npg){
	int i;
	char* c=(char*)start;
	for(i=0;i<npg;i++,c+=PAGE_SIZE) pgRelease(c);
}
void free_mem(mem_t* mem){
	if(mem->npage<=0) return;
	DEBUG_MEM(mem);
	pgRelease_c(mem->page,mem->npage);
	printk("\n Yahoo....");
	docli();
	gdtRemove(mem->sel);
	dosti();
	mem->npage=-1;
	mem->sel=0;
	printk("\n Yogi bear");
}
/* Allocate a number of continous pages */
ptr_t pgAlloc_c(count_t nPg)
{
	/*printk("\n Trying to alloc a %d continuous pages ",nPg); */
	/*printk("\nCalling pgAlloc_c to alloc %d pages",nPg); */
	count_t n = 0;
	count_t x = 0;
	count_t found = 0;
	byte_t* p = (byte_t*)BITMAP_START;
	if(nPg<=0) {panic("Trying to alloc 0 pages !! ");return 0;}	
	/* nStart and xStart are the values of n and x at the start of the group of pages */
	count_t nStart = 0;
	count_t xStart = 0;
	
	/* nTmp and xTmp are used to cycle thru the values of n and x, setting pages as allocated */
	count_t nTmp = 0;
	count_t xTmp = 0;
	ptr_t ptr;
	while (n < BITMAP_SIZE) /* loop through each byte of the page table */
	{
		x = 0;
		while (x < 8)
		{
			if (!(*p & (1 << x)))	/* check allocated flag */
			{
				if (!found)		/* check if first of group found */
				{
					xStart = x;
					nStart = n;
				}
				
				found++;
				if (found == nPg)
				{
					goto FixPages;
					/* get out of this mess of indented loops!! */
				}
			}
			else
				found = 0;
			++x;
		}
		++n;
		++p;
	}
	panic("Out of memory");
	return 0; /* never actually happens, just gets rid of warnings */
	
FixPages:
	/* fix allocated pages */
	dummy();
	ptr=(ptr_t)(POOL_START + PAGE_SIZE * (xStart + nStart * 8));
	/*printk("\n Allocing from %x , %d pages ",ptr,nPg); */
	nTmp = nStart;
	xTmp = xStart;
	p = ((byte_t*)BITMAP_START)+nStart;
	x=xStart;
	while(nPg>0){
		*p = (*p)|(1<<x); 	
		nPg--;
		x++;
		_ctr_alloc++;
		if(x==8) {x=0;p++;}
	}
/*
	// loop thru previous bytes
	while (nTmp < n)
	{
		while (xTmp < 8)
		{
			*p |= (1 << xTmp);
			++xTmp;
		}
		xTmp = 0;
		++p;
		++nTmp;
	}

	// loop thru last byte
	while (xTmp < x)
	{
		*p |= (1 << xTmp);
		++xTmp;
	}
*/
	/* all pages *should* now be marked as allocated... return ptr*/
	return ptr;
}
