
[BITS 32]

%include "../include/symbol.s"

%include "../include/defs.s"

EXPORT StartWorking
EXPORT docli
EXPORT donop
EXPORT dosti
EXPORT outb
EXPORT inb
EXPORT outb_p
EXPORT inb_p
EXPORT do_nothing
EXPORT switch_to
EXPORT load_tr
EXPORT load_ldtr
EXPORT doBreakOn
EXPORT setupDebugReg
EXPORT disableBreakReg
EXPORT enableBreakReg

IMPORT getLinear
IMPORT Main

[SECTION .text]

align 8
StartWorking:
	jmp Main
	ret ; never gonna happen

; Load IDTR with 6-byte pointer whose address is passed as
; the parameter. 
align 8
docli:
	cli
	ret
align 8
dosti:
	sti
	ret

align 8
donop:
	nop
	ret

align 8
outb:
	push	ebp
	mov	ebp, esp
	push eax
	push edx
	mov	eax,[ebp+8]
	mov	edx,[ebp+12]
	out dx,al
	pop edx
	pop eax
	leave
	ret
	
align 8
inb:
	push	ebp
	mov	ebp, esp
	mov	edx,[ebp+8]
	xor eax,eax
	in al,dx
	leave
	ret
align 8
inb_p:
	push	ebp
	mov	ebp, esp
	mov	edx,[ebp+8]
	xor eax,eax
	in al,dx
	jmp a
a:	jmp b
b:	leave
	ret
align 8
outb_p:
	push ebp
	mov ebp,esp
	mov eax,[ebp+8]
	mov edx,[ebp+12]
	push eax
	push edx
	out dx,al
	jmp c
c:	jmp d
d:	pop edx
	pop eax
	leave
	ret
align 8
do_nothing:
	cli
_halt:  jmp _halt
	sti
	ret
align 8
switch_to:
	push ebp
	mov ebp,esp
	sub esp,8
	mov dword [ebp-4],0
	mov dword [ebp-8],0
	mov edx,[ebp+8]
	mov  word [ebp-4],dx
	jmp far [ebp-8]
	clts
	leave
	ret
	
align 8
load_ldtr:
	push ebp
	mov ebp,esp
	push eax
	mov eax,[ebp+8]
	lldt ax
	pop eax
	leave
	ret
align 8
load_tr:
	push ebp
	mov ebp,esp
	push eax
	mov eax,[ebp+8]
	ltr ax
	pop eax
	leave
	ret

align 8
doBreakOn:
	push ebp
	mov ebp,esp
	push ebx
	push edx
	push ecx
	mov edx,[ebp+8]
	mov ebx,[ebp+12]
	;edx the type
	;ebx has the linear address
	mov eax,dr7
	mov ecx,4
next:	test ax,0x3;check if any of the last two bits are set
	jz break0 ;if they are then a dr(4-cx) is already used
	shr ax,2
	loop next
	jmp not_free
	;debug register to use = DRx (where x=(4-ecx))
	
break0:	;using dr0
	cmp ecx,4
	jne break1
	mov dr0,ebx		;mov linear address to dr0
	mov eax,dr7
	or eax,0x200 ;set ge 
	or eax,0x2 		;set global enable 0
	shl edx,16
	or eax,edx ;set the type field 
	cmp edx,0		;is the breakpoint of type instr
	je instr0
	or eax,0xc0000 ;set the len field to 2'b11 , 4 bytes
	jmp breakdone
instr0: and eax,0xfff3ffff ;clear ge,le,len0
	jmp breakdone
	
break1:	;using dr1
	cmp ecx,3
	jne break2
	mov dr1,ebx		;mov linear address to dr1
	mov eax,dr7
	or eax,0x200 ;set ge 
	or eax,0x8 		;set global enable 1
	shl edx,20
	or eax,edx ;set the type field 
	cmp edx,0		;is the breakpoint of type instr
	je instr1
	or eax,0xc00000 ;set the len field to 2'b11 , 4 bytes
	jmp breakdone
instr1: and eax,0xff3fffff ;clear ge,le,len1
	jmp breakdone

break2:	;using dr2
	cmp ecx,2
	jne break3
	mov dr2,ebx		;mov linear address to dr0
	mov eax,dr7
	or eax,0x200 ;set ge 
	or eax,0x20 		;set global enable 0
	shl edx,24
	or eax,edx ;set the type field 
	cmp edx,0		;is the breakpoint of type instr
	je instr2
	or eax,0xc000000 ;set the len field to 2'b11 , 4 bytes
	jmp breakdone
instr2: and eax,0xf3ffffff ;clear ge,le,len2
	jmp breakdone

break3:	;using dr3
	cmp ecx,1
	jne not_free
	mov dr3,ebx		;mov linear address to dr0
	mov eax,dr7
	or eax,0x200 ;set ge 
	or eax,0x80 		;set global enable 0
	shl edx,28
	or eax,edx ;set the type field 
	cmp edx,0		;is the breakpoint of type instr
	je instr3
	or eax,0xc0000000 ;set the len field to 2'b11 , 4 bytes
	jmp breakdone
instr3: and eax,0x3fffffff ;clear ge,le,len3
	jmp breakdone

breakdone: mov dr7,eax
	xor eax,eax
	mov dr6,eax
	mov eax,4
	sub eax,ecx
	jmp breakexit
not_free: xor eax,eax
	mov dr6,eax
	mov eax,-1;eax=-1 in case of error , ie no debug reg's are free
breakexit:	
	pop ecx
	pop edx
	pop ebx
	leave
	ret

align 8
setupDebugReg:
	push ebp
	mov ebp,esp
	push eax
	xor eax,eax
	mov dr7,eax
	mov dr6,eax;no breakpoints set right now
	pop eax
	leave
	ret
	
align 8
enableBreakReg:
	push ebp
	mov ebp,esp
	push ecx
	push ebx
	mov ecx,[ebp+8]
	mov eax,0x2
	shl ecx,1
	shl eax,cl
	mov ebx,dr7
	or ebx,eax
	mov dr7,ebx
	pop ebx
	pop ecx
	leave
	ret
	
align 8
disableBreakReg:
	push ebp
	mov ebp,esp
	push ecx
	push ebx
	mov ecx,[ebp+8]
	mov eax,0x2
	shl ecx,1;multiply ecx by 2 , hence 0,2,4,6
	shl eax,cl
	not eax
	mov ebx,dr7
	and ebx,eax
	mov dr7,ebx
	pop ebx
	pop ecx
	leave
	ret
