#include <floppy.h>
/* Floppy IO ports */
const unsigned short IOBase[2] = {0x3F0, 0x370};
const unsigned short	flpRStatusRegisterA = 0,		FloppyStatusRegisterB = 1,
	flpWDigitalOutputRegister = 2,	flpRMainStatusRegister = 4,
	flpWDataRateSelect = 4,		flpRWDataRegister = 5,
	flpRDigitalInputRegister = 7,	flpWConfigurationControlRegister = 7;

const unsigned int MaximumReadWriteTries = 5, MaximumSeekTries = 2, MaximumRecalibrateTries = 5;
const unsigned int MaximumSendTries = 10000, MaximumGetTries = 10000;

const unsigned int WaitDelay = 75, SeekDelay = 15, MotorOnDelay = 500;
/* Variables ----------------------------------------------------------------------------------- */
byte_t ControllerIndex = 0, DriveIndex = 0;	/* Default to the first drive of the first controller. Currently this is the only place to set the controller index; if you want to support both controllers, you will want to add code for setting the controller index. */
enum DriveType Type = drvNone;		/* Assume no drive until told otherwise. */

byte_t AtCylinder = 255;

byte_t MotorRunning = 0, DiskChanged = 0;

/* Floppy drive status registers */
byte_t DigitalOutputRegister = dorNoReset | dorDMA, StatusRegister0 = 0, StatusRegister1 = 0, StatusRegister2 = 0;

volatile byte_t PendingInterrupt = 0;
/*default values being 0,4 */
extern  void FloppyInterrupt();
void InitFloppy() {
	DriveIndex = (byte_t)DRIVE_INDEX;
	Type = (enum DriveType)4;
	InstallHandler(IDT_FIRST_EXTERNAL + 6, &FloppyInterrupt);
}


byte_t ReadSectorStatus(byte_t Head, byte_t Sector, byte_t SectorSize) {
	byte_t TestCylinder, TestHead, TestSector, TestSectorSize;

	if (GetData(&StatusRegister0)) {
		if (GetData(&StatusRegister1)) {
			if (GetData(&StatusRegister2)) {
				if (GetData(&TestCylinder)) {
					if (GetData(&TestHead)) {
						if (GetData(&TestSector)) {
							if (GetData(&TestSectorSize)) {
								return (TestCylinder == AtCylinder) && (TestHead == Head) && (TestSector == Sector+1) && (TestSectorSize == SectorSize);
							}
						}
					}
				}
			}
		}
	}

	return 0;
}

	byte_t ReadStatus() {
		if (!SendData(cmdCheckInterruptStatus))
			return 0;
		if (!GetData(&StatusRegister0))
			return 0;
		if(!GetData(&AtCylinder))
			return 0;

		return 1;
	}

byte_t SendData(byte_t Data) {
	unsigned int Tries;	
	for (Tries = 0; Tries < MaximumSendTries; Tries++) {
		if ((ReadByteFromPort(IOBase[ControllerIndex] + flpRMainStatusRegister) & (msrMainRequest | msrWaitingForDataIO)) >= msrMainRequest) {
			WriteByteToPort(IOBase[ControllerIndex] + flpRWDataRegister, Data);
			return 1;
		}

		SmallDeltaDelay
	}

	return 0;
}

byte_t GetData(byte_t*  Data) {
	unsigned int Tries;
	for (Tries = 0; Tries < MaximumGetTries; Tries++) {
		if ((ReadByteFromPort(IOBase[ControllerIndex] + flpRMainStatusRegister) & (msrMainRequest | msrWaitingForDataIO)) == (msrMainRequest | msrWaitingForDataIO)) {
			*Data = ReadByteFromPort(IOBase[ControllerIndex] + flpRWDataRegister);
			return 1;
		}

		SmallDeltaDelay
	}

	return 0;
}

void DoFloppyInterrupt() {
	PendingInterrupt = 0;
}
void Start() {
	EnableIRQ(6,1);
	Reset();

	StartMotor();

	/* Now get the floppy drive ready */
	SetDataRate(ccrDataRate500Kb);
	SetSpecifications(spcStepRate3ms, spcHeadUnloadTime240ms, spcHeadLoadTime16ms);
	Recalibrate();
}


void Reset() {
	/* Reset the controller */
	PendingInterrupt = 1;
	WriteByteToPort(IOBase[ControllerIndex] + flpWDigitalOutputRegister, 0);
/*	while(PendingInterrupt); */
	SmallDeltaDelay
	printk("\n First Hurdle clear");
}

void SetDataRate(enum CCR DataRate) {
	WriteByteToPort(IOBase[ControllerIndex] + flpWConfigurationControlRegister, DataRate);
}

byte_t SetSpecifications(byte_t StepRate, byte_t HeadUnloadTime, byte_t HeadLoadTime) {
	if (!SendData(cmdSpecify)) return 0;
	if (!SendData((StepRate << spcStepRateShift) & HeadUnloadTime)) return 0;
	if (!SendData(HeadLoadTime << spcHeadLoadTimeShift)) return 0;

	return 1;
}

byte_t Recalibrate() {
	unsigned int Tries = 0;

	AtCylinder = 255;
	while ((Tries < MaximumRecalibrateTries) && (AtCylinder != 0)) {
		PendingInterrupt = 1;

		SendData(cmdCalibrateDrive);
		SendData(DriveIndex);

		#ifndef _BOCHS_
		while(PendingInterrupt);
		#else 
		SmallDeltaDelay
		#endif
		

		ReadStatus();

		Tries++;
	}

	return Tries < MaximumRecalibrateTries;
}


void StartMotor() {
	if (!MotorRunning) {
		PendingInterrupt = 1;

		DigitalOutputRegister |= (1 << (DriveIndex + dorMotorShift)) | DriveIndex;
		WriteByteToPort(IOBase[ControllerIndex] + flpWDigitalOutputRegister, DigitalOutputRegister);

/*		while(PendingInterrupt); */
		SmallDeltaDelay

		MotorRunning = 1;
	}
}
byte_t SeekCylinder(byte_t Head, byte_t Cylinder) {
	unsigned int Tries = 0;

	while ((Tries < MaximumSeekTries) && (AtCylinder != Cylinder)) {
		PendingInterrupt = 1;

		if (SendData(cmdSeek)) {
			if (SendData((Head << cmdHeadShift) | DriveIndex)) {
				if (SendData(Cylinder)) {
					while(PendingInterrupt);

					if (!ReadStatus()) {
						AtCylinder = 255;		/* Make sure we don't assume the cylinder is correct, since ReadStatus failed. */
					}
				}
			}
		}

		Tries++;
	}

	return AtCylinder == Cylinder;
}

void SetupDMA(count_t count)
{
	iptr_t addr = DMA_START_ADDR;	/* 576k */

	/* assume count is correctly set up to be less than 64k */

	docli();	/* disable interrupts before we start */

/* disable DMA 2 just to be sure we are in control */
	outb_p(6,10);
/* output command byte twice */
 	__asm__("outb %%al,$12\n\tjmp 1f\n1:\tjmp 1f\n1:\t"
	"outb %%al,$11\n\tjmp 1f\n1:\tjmp 1f\n1:"::
	"a" ((char) 0x46));			/* hard coded to read from the floppy */
/* 8 low bits of addr */
	outb_p(addr & 0xFF,4);
	addr >>= 8;
/* bits 8-15 of addr */
	outb_p(addr & 0xFF,4);
	addr >>= 8;
/* bits 16-19 of addr */
	outb_p(addr & 0x0F,0x81);

	--count;
/* low 8 bits of count-1 */
	outb_p(count & 0xFF,5);
	count >>= 8;
/* high 8 bits of count-1 */
	outb_p(count & 0xFF,5);

/* activate DMA 2 */
	outb_p(2,10);

	/* re-enable interrupts */
	dosti();
}

/* this is one of mine */
void EndDMA()
{
	outb_p(6, 10);
}




byte_t FloppyRead(unsigned int BlockIndex) {
	unsigned int Head, Cylinder, Sector;
	unsigned int Tries = 0, Continue = 1;

	Sector = BlockIndex % 18 + 1;
	Head = (BlockIndex / 18) % 2;
	Cylinder = BlockIndex / 36;

	SetupDMA(512);
	printk("\n DMA Setup .. ");

	Start();	/* Start the motor and set everything up. */
	printk("\n I setup Properly ");
	while ((Tries < MaximumReadWriteTries) && Continue) {
		if (SeekCylinder(Head, Cylinder)) {
			PendingInterrupt = 1;

			if (SendData(cmdReadSector | cmdMultitrack | cmdDoubleDensityMode | cmdSkipDeletedSectors)) {
				if (SendData((Head << cmdHeadShift) | DriveIndex)) {
					if (SendData(Cylinder)) {
						if (SendData(Head)) {
							if (SendData(Sector)) {
								if (SendData(2)) {	/* Sector size = 128 * pow(2, <argument to SendData>)*/
									if (SendData(Sector)) {
										if (SendData(cmdGAP3Length)) {
											if (SendData(255)) {		/* Doesn't matter unless the sector size is 128 byte_ts */
												while(PendingInterrupt);
												Continue = !ReadSectorStatus(Head, Sector, 2);
											}
										}
									}
								}
							}
						}
					}
				}
			}
		}

		if (Continue)
			Recalibrate();

		Tries++;
	}

	Stop();
	EndDMA();
	return (Tries < MaximumReadWriteTries); /*0 on unsuccefull , 1 on successfull  */
}

void Stop() {
	StopMotor();
	EnableIRQ(6,0);
}


void StopMotor() {
	if (MotorRunning) {
		DigitalOutputRegister &= 0xFC - (1 << (DriveIndex + dorMotorShift));
		WriteByteToPort(IOBase[ControllerIndex] + flpWDigitalOutputRegister, DigitalOutputRegister);
		/* Since not all floppy drive controllers raise an interrupt when the motor has stopped, we won't wait. */

		AtCylinder = 255;		/* The cylinder we were at is no longer valid, since the motor has stopped. */
		MotorRunning = 0;
	}
}


