
#include <stdtypes.h>

#include <gdt.h>
#include <video.h>
#include <kalloc.h>
/* basic size of 128 entries, which is stupidly large */

/* note this places a 1k limit on setup code which wont be overwritten */
/*   the GDT in setup *must* be kept below this limit*/

/* declare this privately here */
gdtEntry* gdtGetFree(byte_t* pEntry);
mem_t dummyLdt;
void InitLDT(){
	/*int dldt; */
	/*dldt=DUMMY_LDT; */
	dummyLdt.page=(void*)(LDTBASE);/*pgAlloc(); */
	dummyLdt.npage=1;
	dummyLdt.sel=DUMMY_LDT;
	gdtSet(3,0x2,(dword_t)(dummyLdt.page),dummyLdt.npage*PAGE_SIZE,0,1,1,0);
	load_ldtr((DUMMY_LDT&0xffff));
	/*asm("lldt %0"::"a"(dldt)); */
}
/* initialise the kernel GDT */
void InitGDT()
{
    word_t limitAndBase[3];
	gdtEntry* desc;
	count_t i;

	VidWrite("Initialising GDT... ");

	/* firstly, wipe our GDT for safety */
	memset((ptr_t)GDT_START, (byte_t)0, GDT_LIMIT);

	/* initialise the kernel code segment */
	gdtSet(1, 0x0A,0,KFULL,1,1,1,0);
	gdtSet(2, 0x02,0,KFULL,1,1,1,0);
	/* Resv the ldt,ktss,threadTss entries */
	((gdtEntry*)(GDT_START + (KERNEL_TSS_IDX*8 )))->present=1;
	((gdtEntry*)(GDT_START + (THREAD_TSS_IDX *8)))->present=1;
	((gdtEntry*)(GDT_START + (LDT_IDX *8)))->present=1;
	/*the third one will be set later */
	/* now set every other entry as available */
	/*the third one is the ldt dummy */
	i = 6;	/* start with 6th entry */
	while (i < GDT_NUM_ENTRIES)
	{
		desc = (gdtEntry*)(GDT_START + (8 * i++));
		desc->present= 0;
	}

	/* cruft together a limit and base, then load the GDT. */
	/*   This bit is taken from GeekOS, with modifications */
	limitAndBase[0] = GDT_LIMIT;
    limitAndBase[1] = GDT_START & 0xffff;
    limitAndBase[2] = GDT_START >> 16;
    gdtLoad(limitAndBase);

	VidWrite("Done\n");
}

/*///////////////////////////////////////////////////////////////// */
/* new functions to manage the GDT */

/* add an entry, returns the entry number */
/* assume interrupts are off otherwise we might have problems */
byte_t gdtAdd(byte_t type,
			ulong_t base /* = 0 */,
			ulong_t size /* = 0xFFFFF */,
			byte_t system /* = 1 */,
			byte_t granularity /* = 1 */,
			byte_t dbBit /* = 1 */,
			byte_t dpl /* = 0 */)
{
	byte_t ret;
	gdtEntry* desc = gdtGetFree(&ret);
	if (desc == NULL)
		return 0xFF;	/* currently 255th entry doesnt exist */
	gdtSetSize(desc, size);
	gdtSetBase(desc, base);
	desc->system = system;
	desc->present = 1;
	desc->granularity = granularity;
	desc->type = type;
	desc->dbBit = dbBit;
	desc->dpl = dpl;
	desc->avail = 0;
	
	return ret;
}

/* get an existing entry */
gdtEntry* gdtGet(byte_t entry)
{
	return (gdtEntry*)(GDT_START + (entry<<3));
}

/* copy an entry into the table */
/* I expect this will rarely be used, but could reduce the amount of time */
/*    interrupts will be disabled for once memcpy() is optimised. */
void gdtPut(byte_t entry, gdtEntry* desc)
{
	memcpy(gdtGet(entry), desc, sizeof(gdtEntry));
}

/* remove an entry */
/* again interrupts should be disabled, we always assume this will work */
void gdtRemove(byte_t entry)
{
	printk("\n Asking me to remove entry no %d",entry);
	gdtEntry* desc = gdtGet(entry);
	memset((ptr_t)desc, 0, sizeof(gdtEntry));
	desc->present=0;
	printk("\n Removed");
}

/* modify an entry, or add an entry at a certain position */
/* assume interrupts are disabled yet again */
void gdtSet(byte_t entry,
			byte_t type,
			ulong_t base /* = 0 */,
			ulong_t size /* = 0xFFFFF */,
			byte_t system /* = 1 */,
			byte_t granularity /* = 1 */,
			byte_t dbBit /* = 1 */,
			byte_t dpl /* = 0 */)
{
	gdtEntry* desc = gdtGet(entry);
	memset((ptr_t)desc, 0, sizeof(gdtEntry));
	gdtSetSize(desc, size);
	gdtSetBase(desc, base);
	desc->system = system;
	desc->present = 1;
	desc->granularity = granularity;
	desc->type = type;
	desc->dbBit = dbBit;
	desc->dpl = dpl;
	desc->avail = 0;
}

/* loop through the gdt looking for a free entry */
gdtEntry* gdtGetFree(byte_t* entry)
{
	gdtEntry* desc;
	count_t i = 1;	/* start with 1st entry, since 0 is left blank */
	
	while (i < GDT_NUM_ENTRIES)
	{
		desc = gdtGet(i);
		if (desc->present==0)
		{
			*entry = i;
			return desc;
		}
		++i;
	}
	
	/* there doesn't seem to be any free GDT entries! */
	*entry = 0xFF;
	return (gdtEntry*)NULL;
}
dword_t getLinear (dword_t selector, dword_t offset)
{
	selector&=0xffff;
	if (selector == KERNEL_CS || selector == KERNEL_DS) return offset;
	else {
		gdtEntry* g=gdtGet(selector>>3);
		return GDTBASE(g)+offset;
	}
}	

