
#include <stdtypes.h>

#include <idt.h>
#include <defs.h>
#include <kalloc.h>
#include <video.h>
#include <asm.h>
#include <sys.h>
#include <debugger.h>
/* size of IDT */
#define IDT_SIZE (IDT_NENTRIES << 3)

/* start the IDT at 6k */
char* intToString[]={"Div Err","Debug","NMI","BreakPoint","Overflow","Bound","Invalid Opcode","Device Unavailable","Double Fault","387 Overrun","Invalid TSS","Segment absent","SF","GPF","Page Fault","Reserved!","387 error"};
/* the default IDT is 2k in size, so spans from 6k to 8k */

extern  void DummyInterrupt();

/* interrupts must be disabled when calling this */
void PrivateInstallHandler(idtEntry* desc, InterruptHandler hand)
{
	iptr_t addr = (iptr_t)hand;
	int index=((long)((void*)desc)-IDT_START)>>3;	
	desc->ig.offsetLow = addr & 0xffff;
	desc->ig.segmentSelector = KERNEL_CS;
	desc->ig.reserved = 0;
	desc->ig.signature = 0x70; /* == 01110000b */
	desc->ig.dpl = (index<17)?3:0;
	desc->ig.present = 1;
	desc->ig.offsetHigh = addr >> 16;
}
extern  void MaskAllIrq();
/* initialise the idt */
extern  void DefInterrupt0();
extern  void DefInterrupt1();
extern  void DefInterrupt2();
extern  void DefInterrupt3();
extern  void DefInterrupt4();
extern  void DefInterrupt5();
extern  void DefInterrupt6();
extern  void DefInterrupt7();
extern  void DefInterrupt8();
extern  void DefInterrupt9();
extern  void DefInterrupt10();
extern  void DefInterrupt11();
extern  void DefInterrupt12();
extern  void DefInterrupt13();
extern  void DefInterrupt14();
extern  void DefInterrupt15();
extern  void DefInterrupt16();
extern  void DefInterrupt17();
extern  void DefInterrupt18();
extern  void DefInterrupt19();
/* called by the dummy handler */
extern  void DoDummyInterrupt()
{
	panic("**** really *** Bad Interrupt");
}
extern  void DefInt(int x)
{
	PrintHex(x);
	if(fromThread) {
		printk("\n Encountered a %s at %x:%x ,program termintated",(x<17?intToString[x]:"Unknown fault"),CS(progData),EIP(progData));
		kill();
		switch_to(ktss.sel<<3);
	}
	else panic("*** Bad Interrupt");
}
void InitIDT()
{
    word_t limitAndBase[3];
	count_t i = 0;

	VidWrite("Initialising IDT... ");

    /* Blank out the IDT before we start just for safety */
	memset((ptr_t)IDT_START, (byte_t)0, IDT_SIZE);
	
	PrivateInstallHandler((idtEntry*)(IDT_START + (0 << 3)), &DefInterrupt0);
	PrivateInstallHandler((idtEntry*)(IDT_START + (1 << 3)), &DefInterrupt1);
	PrivateInstallHandler((idtEntry*)(IDT_START + (2 << 3)), &DefInterrupt2);
	PrivateInstallHandler((idtEntry*)(IDT_START + (3 << 3)), &DefInterrupt3);
	PrivateInstallHandler((idtEntry*)(IDT_START + (4 << 3)), &DefInterrupt4);
	PrivateInstallHandler((idtEntry*)(IDT_START + (5 << 3)), &DefInterrupt5);
	PrivateInstallHandler((idtEntry*)(IDT_START + (6 << 3)), &DefInterrupt6);
	PrivateInstallHandler((idtEntry*)(IDT_START + (7 << 3)), &DefInterrupt7);
	PrivateInstallHandler((idtEntry*)(IDT_START + (8 << 3)), &DefInterrupt8);
	PrivateInstallHandler((idtEntry*)(IDT_START + (9 << 3)), &DefInterrupt9);
	PrivateInstallHandler((idtEntry*)(IDT_START + (10 << 3)), &DefInterrupt10);
	PrivateInstallHandler((idtEntry*)(IDT_START + (11 << 3)), &DefInterrupt11);
	PrivateInstallHandler((idtEntry*)(IDT_START + (12 << 3)), &DefInterrupt12);
	PrivateInstallHandler((idtEntry*)(IDT_START + (13 << 3)), &DefInterrupt13);
	PrivateInstallHandler((idtEntry*)(IDT_START + (14 << 3)), &DefInterrupt14);
	PrivateInstallHandler((idtEntry*)(IDT_START + (15 << 3)), &DefInterrupt15);
	PrivateInstallHandler((idtEntry*)(IDT_START + (16 << 3)), &DefInterrupt16);
	PrivateInstallHandler((idtEntry*)(IDT_START + (17 << 3)), &DefInterrupt17);
	PrivateInstallHandler((idtEntry*)(IDT_START + (18 << 3)), &DefInterrupt18);
	PrivateInstallHandler((idtEntry*)(IDT_START + (19 << 3)), &DefInterrupt19);
	i=20;
	/* load dummy interrupt handlers */
	while (i < IDT_NENTRIES){
		PrivateInstallHandler((idtEntry*)(IDT_START + (i << 3)), &DoDummyInterrupt);
		/*PrivateInstallHandler((idtEntry*)(IDT_START + (i << 3)), &(DefInterrupt[i]));*/
		i++;
	}

    /* Cruft together a 16 bit limit and 32 bit base address */
    /* to load into the IDTR. */
    limitAndBase[0] = IDT_SIZE;
    limitAndBase[1] = IDT_START & 0xffff;
    limitAndBase[2] = IDT_START >> 16;

    /* Install the new table in the IDTR. */
    idtLoad(limitAndBase);

	VidWrite("Done\n");
	MaskAllIrq();
}

/* this should be kept up to date with what we load when booting */
static word_t irqMask = 0xFFFB;

#define MASTER(mask) ( (mask) & 0xff )
#define SLAVE(mask) ( ((mask)>>8) & 0xff )

void EnableIRQ(byte_t irq, bool_t enable)
{
	byte_t oldMask, newMask, mask;

	if (enable)
		mask = irqMask & ~(1 << irq);
	else
		mask = irqMask | (1 << irq);

    oldMask = MASTER(irqMask);
    newMask = MASTER(mask);
    if ( newMask != oldMask )
	{
		outb(newMask, 0x21);
    }

    oldMask = SLAVE(irqMask);
    newMask = SLAVE(mask);
    if ( newMask != oldMask )
	{
		outb(newMask, 0xA1);
    }

    irqMask = mask;
}
/* public function to install an interrupt handler, */
/*  interrupts must be disabled? */
void InstallHandler(byte_t i, InterruptHandler hand)
{
	PrivateInstallHandler((idtEntry*)(IDT_START + (i << 3)), hand);
}

