
[BITS 32]

%include "../include/symbol.s"

; include this for KERNEL_CS
%include "../include/defs.s"

EXPORT idtLoad
EXPORT gdtLoad


EXPORT KeyboardInterrupt
EXPORT FloppyInterrupt
EXPORT DebuggerInterrupt


IMPORT ProcessKeycode
IMPORT doDebuggerInt
IMPORT progData
IMPORT DefInt
IMPORT getLinear
IMPORT DoFloppyInterrupt
IMPORT displayTss
IMPORT doFullSaveState 

;note that ss , cs, eflags,eip are not saved , if need to be saved ,that is done by the 386 itself.
%macro save_state 0
	nop
	nop
	pushad ; ax,cx,dx,bx,sp old,bp,si,di  in that order
		;pushing segments in this order ds,es,fs,gs,ss
	xor eax,eax
	mov ax,ds
	push eax
	
	xor eax,eax
	mov ax,es
	push eax
	
	xor eax,eax
	mov ax,fs
	push eax
	
	xor eax,eax
	mov ax,gs
	push eax	

	xor eax,eax
	mov ax,ss
	push eax	
	
%endmacro
%macro  move_to_kernel_ds 0
	mov ax,KERNEL_DS
	mov ds,ax
	mov es,ax
	mov fs,ax
	mov gs,ax
%endmacro
%macro restore_state 0
	pop eax
	mov gs,ax
	
	pop eax
	mov fs,ax
	
	pop eax
	mov es,ax
	
	pop eax
	mov ds,ax
	add esp,4;gloss over the ss stored here. 
	popad
%endmacro

%macro full_saveState 1
	save_state
	move_to_kernel_ds
	push esp	;points to the just pushed in gs above
	xor eax,eax
	mov ax,ss
	push eax
	xor eax,eax
	str ax
	push eax
	push %1
	call doFullSaveState ;doFullSaveState(intNum,tr,ss,ss_offset)
	;the above function shall write into the progData , and the eip_idx etc var's 
	add esp,16	
%endmacro
%macro _DefInterrupt 1
EXPORT DefInterrupt%1
align 8
DefInterrupt%1:
	full_saveState %1
	push %1
	call DefInt
	add esp,4
	restore_state
	iret
%endmacro
; ----------------------------------------------------------------------
; Code
; ----------------------------------------------------------------------

[SECTION .text]


; Load IDTR with 6-byte pointer whose address is passed as
; the parameter. 
align 8
idtLoad:
	mov	eax, [esp+4]
	lidt [eax]
	ret

; Load GDTR with 6-byte pointer whose address is passed as
; the parameter. This really belongs somewhere else, but for
; now it is here.
align 8
gdtLoad:
	mov eax, [esp+4]
	lgdt [eax]

	; we know that all the segment registers are
	; correct because they were set during setup

	; now do a far jump to reload everything
	jmp KERNEL_CS:.here

.here:
	ret
	
%assign x 0 
%rep   20 
	_DefInterrupt x
	%assign x x+1 
%endrep
	
align 8
KeyboardInterrupt:
	push ax
	push dx
	mov dx, 0x64
	xor ax, ax
	in al, dx		; get keyboard status
	and al, 1		; check if there is some data ready
	jz .done

	mov dx, 0x60
	xor ax, ax
	in al, dx		; get data
	push ax			; push onto stack as an argument
	call ProcessKeycode
	pop ax			; put things back how they were

.done
	mov dx, 0x20
	mov al, 0x61
	out dx, al		; EOI to master PIC

	pop dx
	pop ax
	iret

align 8
FloppyInterrupt:
	save_state
	move_to_kernel_ds
	call DoFloppyInterrupt
	mov dx, 0x20
	mov al, 0x66	; IRQ 6
	out dx, al		; EOI to master PIC
	restore_state
	iret
align 8
DebuggerInterrupt:
		;stack before has ss,esp,eflags,cs,eip in this order 
		; the subsequent sp value pushed hence points to the eip
	
	full_saveState 1	
	mov eax,dr6 ; save debug reg
	push ebx
	xor ebx,ebx
	mov dr6,ebx	;clear dr6
	pop ebx
	push eax; this will be some garbage right now
	call doDebuggerInt ; this does some printing and switches to the kernel task .
	add esp,4
	restore_state	
	iret ; this shll switch back to the original stack.

