/*Make sure this has no dependencies or other files */
/*
 *  linux/kernel/printk.c
 *
 *  (C) 1991  Linus Torvalds
 */

/*
 * When in kernel-mode, we cannot use printf, as fs is liable to
 * point to 'interesting' things. Make a printf with fs-saving, and
 * all is well.
 */
/*static char buf[1024]; //this being a static buf , where does this lie in the kernel ? or should I specify that ? */

/* we use this so that we can do without the ctype library */
#include <printk.h>
#include <ascii.h>
#include <string.h>
#define is_digit(c)	((c) >= '0' && (c) <= '9')
static int skip_atoi(const char **s)
{
	int i=0;

	while (is_digit(**s))
		i = i*10 + *((*s)++) - '0';
	return i;
}
fileh conDev;
#define ZEROPAD	1		/* pad with zero */
#define SIGN	2		/* unsigned/signed long */
#define PLUS	4		/* show plus */
#define SPACE	8		/* space if plus */
#define LEFT	16		/* left justified */
#define SPECIAL	32		/* 0x */
#define SMALL	64		/* use 'abcdef' instead of 'ABCDEF' */

const char *digits="0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ";
char tmp[36];
static char * number(char * str, int num, int base, int size, int precision
	,int type)
{
	char c,sign;
	int i;

	if (type&LEFT) type &= ~ZEROPAD;
	if (base<2 || base>36)
		return 0;
	c = (type & ZEROPAD) ? '0' : ' ' ;
	if (type&SIGN && num<0) {
		sign='-';
		num = -num;
	} else
		sign=(type&PLUS) ? '+' : ((type&SPACE) ? ' ' : 0);
	if (sign) size--;
	if (type&SPECIAL)
		if (base==16) size -= 2;
		else if (base==8) size--;
	i=0;
	if (num==0)
		tmp[i++]='0';
	else while (num!=0){tmp[i++]=digits[num%base];num/=base;}
	if (i>precision) precision=i;
	size -= precision;
	if (!(type&(ZEROPAD+LEFT)))
		while(size-->0)
			*str++ = ' ';
	if (sign)
		*str++ = sign;
	if (type&SPECIAL)
		if (base==8)
			*str++ = '0';
		else if (base==16) {
			*str++ = '0';
			*str++ = digits[33];
		}
	if (!(type&LEFT))
		while(size-->0)
			*str++ = c;
	while(i<precision--)
		*str++ = '0';
	while(i-->0)
		*str++ = tmp[i];
	while(size-->0)
		*str++ = ' ';
	return str;
}
int len;
int i;
char * str;
char *s;
int *ip;

int flags;		/* flags to number() */

int field_width;	/* width of output field */
int precision;		/* min. # of digits for integers; max
			   number of chars for from string */
int qualifier;		/* 'h', 'l', or 'L' for integer fields */

int vsprintf(char *buf,const char* fmt,va_list args )
{
	len=0;
	i=0;
	str=0;
	s=0;
	ip=0;

	flags=0;		/* flags to number() */

	field_width=0;	/* width of output field */
	precision=0;		/* min. # of digits for integers; max
				   number of chars for from string */
	qualifier=0;		/* 'h', 'l', or 'L' for integer fields */
	for (str=buf ; *fmt ; ++fmt) {
		if (*fmt != '%') {
			*str++ = *fmt;
			continue;
		}

		/* process flags */
		flags = 0;
repeat:
		++fmt;		/* this also skips first '%' */
		switch (*fmt) {
			case '-': flags |= LEFT; goto repeat;
			case '+': flags |= PLUS; goto repeat;
			case ' ': flags |= SPACE; goto repeat;
			case '#': flags |= SPECIAL; goto repeat;
			case '0': flags |= ZEROPAD; goto repeat;
		}

		/* get field width */
		field_width = -1;
		if (is_digit(*fmt))
			field_width = skip_atoi(&fmt);
		else if (*fmt == '*') {
			/* it's the next argument */
			field_width = va_arg(args, int);
			if (field_width < 0) {
				field_width = -field_width;
				flags |= LEFT;
			}
		}

		/* get the precision */
		precision = -1;
		if (*fmt == '.') {
			++fmt;	
			if (is_digit(*fmt))
				precision = skip_atoi(&fmt);
			else if (*fmt == '*') {
				/* it's the next argument */
				precision = va_arg(args, int);
			}
			if (precision < 0)
				precision = 0;
		}

		/* get the conversion qualifier */
		qualifier = -1;
		if (*fmt == 'h' || *fmt == 'l' || *fmt == 'L') {
			qualifier = *fmt;
			++fmt;
		}

		switch (*fmt) {
			case 'c':
				if (!(flags & LEFT))
					while (--field_width > 0)
						*str++ = ' ';
				*str++ = (unsigned char) va_arg(args, int);
				while (--field_width > 0)
					*str++ = ' ';
				break;

			case 's':
				s = va_arg(args, char *);
				len = strlen(s);
				if (precision < 0)
					precision = len;
				else if (len > precision)
					len = precision;

				if (!(flags & LEFT))
					while (len < field_width--)
						*str++ = ' ';
				for (i = 0; i < len; ++i)
					*str++ = *s++;
				while (len < field_width--)
					*str++ = ' ';
				break;

			case 'o':
				str = number(str, va_arg(args, unsigned long), 8,
						field_width, precision, flags);
				break;

			case 'p':
				if (field_width == -1) {
					field_width = 8;
					flags |= ZEROPAD;
				}
				str = number(str,
						(unsigned long) va_arg(args, void *), 16,
						field_width, precision, flags);
				break;

			case 'x':
				flags |= SMALL;
			case 'X':
				str = number(str, va_arg(args, unsigned long), 16,
						field_width, precision, flags);
				break;

			case 'd':
			case 'i':
				flags |= SIGN;
			case 'u':
				str = number(str, va_arg(args, unsigned long), 10,
						field_width, precision, flags);
				break;

			case 'n':
				ip = va_arg(args, int *);
				*ip = (str - buf);
				break;

			default:
				if (*fmt != '%')
					*str++ = '%';
				if (*fmt)
					*str++ = *fmt;
				else
					--fmt;
				break;
		}
	}
	*str = '\0';
	return str-buf;
}
void init_printk(){
	conDev=OpenDevice(CONSOLE_DEVICE);
}
void clearScreen(){ ioctl(conDev,CON_CLEAR,NULL);}
int printk(const char *fmt, ...)
{
	va_list args;
	va_start(args, fmt);
/*	char* buf=(char*)(pgAlloc()); */
	char* buf=(char*)(PRINTK_BUF_START);
	vsprintf(buf,fmt,args);
	va_end(args);
	while(*buf){
		write(conDev,*buf);
		buf++;
	}
/*	pgRelease((void*)buf);	*/
	return 1;
}

void putString(char c,char* buf,int* head)
{
		switch (c)
		{
				
			case ASCII_LF:		/* line feed (new line) */
				buf[(*(head))++]='\n';
	/*			printk("\n Matched a line feed"); */
				break;
			case ASCII_BS:		/* backspace */
				if(*head) (*head)--;
	/*			printk("\n Matched a backspace"); */
				break;
			case ASCII_HT:		/* horizontal tab */
				break;
			default:	
				if(isnotctrl(c)) {
					buf[(*head)++]=c;
				}
		}
}

void gets(char* buf,int n){
	/*read uptill a \n or a till n chars  */
	int i=0;
	/*need to handle the echoing thing . */
	char c=(char)0;
	int head=0;
	int _cx=cx;
	while(c!='\n' && head<n-1){
		if(read(conDev,&c)!=DEV_SUCCESS) continue;
		if(c==ASCII_BS){
			if(cx>_cx) {
				write(conDev,c);
				putString(c,buf,&head);
			}
		}else {
			write(conDev,c);
			putString(c,buf,&head);
		}
	}
	buf[head-1]=0; /*pack the newline at the end */
}
void scanf(char* fmt, ...){
	va_list params;
	va_start(params,fmt);
	char* buf=(char*)SCANF_BUF_START;
	gets(buf,PAGE_SIZE>>1);
	strcat(buf," By Gets\n");
	VidWrite(buf);
	vsscanf(buf,fmt,params);
	va_end(params);
}

