
/*
	old video code moved to vidraw.c, definitions moved to video.h
*/

#include <stdtypes.h>

#include <device.h>
#include <video.h>
#include <kalloc.h>
#include <asm.h>
#include <sys.h>

/* record based write function */
int_t VidPut(fileh f, void* pBuf, ulong_t rec)
{
	if (rec >= VID_NRECORDS)
		return DEV_ERR_NO_RECORD;
	memcpy((void*)(VID_START + (2 * rec)), pBuf, 2);
	return DEV_SUCCESS;
}

/* read from video memory */
int_t VidGet(fileh f, void* pBuf, ulong_t rec)
{
	if (rec >= VID_NRECORDS)
		return DEV_ERR_NO_RECORD;
	memcpy(pBuf, (void*)(VID_START + (2 * rec)), 2);
	return DEV_SUCCESS;
}

/*///////////////////////////////////////// */
/* Control operations */

void VidSetCursor(unsigned_t pos)
{
    byte_t orig;

    /* Save original contents of CRT address register. */
    orig = inb_p(VID_AREG);

    /* Set the high cursor location byte */
    outb_p(VID_CURSOR_LOC_HIGH_REG, VID_AREG);
    outb_p((pos>>8) & 0xff, VID_DREG);

    /* Set the low cursor location byte */
    outb_p(VID_CURSOR_LOC_LOW_REG, VID_AREG);
    outb_p(pos & 0xff, VID_DREG);

    /* Restore contents of the CRT address register */
    outb(orig, VID_AREG);
}

/* UNTESTED!!! */
void VidGetCursor(unsigned_t* pos)
{
    byte_t orig;

    /* Save original contents of CRT address register. */
    orig = inb_p(VID_AREG);

    /* Get the high cursor location byte */
    outb_p(VID_CURSOR_LOC_HIGH_REG, VID_AREG);
    *pos = (inb_p(VID_DREG) << 8);

    /* Get the low cursor location byte */
    outb_p(VID_CURSOR_LOC_LOW_REG, VID_AREG);
    *pos += inb_p(VID_DREG);

    /* Restore contents of the CRT address register */
    outb(orig, VID_AREG);
}

void VidGetParams(VidParams* p)
{
	p->rows = VID_NROWS;
	p->cols = VID_NCOLS;
}

void VidScrollHelper()
{
	memcpy(VID_START, VID_START + 2*VID_NCOLS, 2*VID_NCOLS*(VID_NROWS-1));
}

/*///////////////////////////////////////// */

int_t VidIoctl(fileh f, ulong_t request, void* arg)
{
	switch (request)
	{
		case VID_SET_CURSOR:
			VidSetCursor((unsigned_t)arg);
			break;
		case VID_GET_CURSOR:
			VidGetCursor((unsigned_t*)arg);
			break;
		case VID_GET_PARAMS:
			VidGetParams((VidParams*)arg);
			break;
		case VID_SCROLL_HELPER:
			VidScrollHelper();
			break;
		default:
			return DEV_ERR_INVALID_REQUEST;
	}
	
	return DEV_SUCCESS;
}

int_t VidOpen(fileh f)
{
	/* nothing to do */
	return DEV_SUCCESS;
}

int_t VidClose(fileh f)
{
	/* again, nothing to do */
	return DEV_SUCCESS;
}

k_dev devVideo;

void InitVideo()
{
	devVideo.flags = 0;
	devVideo.ulRecordSize = 2;
	
	devVideo.fnOpen = VidOpen;
	devVideo.fnClose = VidClose;
	devVideo.fnRead_R = VidGet;
	devVideo.fnWrite_R = VidPut;
	devVideo.fnIoctl = VidIoctl;
	
	RegisterDevice(VIDEO_DEVICE, &devVideo);
}
